import { Injectable } from '@angular/core';
import { Router } from '@angular/router';

import { Effect, Actions, ofType } from '@ngrx/effects';
import { Action } from '@ngrx/store';

import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';

import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/do';
import 'rxjs/add/operator/exhaustMap';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/take';

import { RPCError } from 'packages/core/rpc/error';

import { Notification } from '../../model';
import { NotificationService } from '../../service/notification.service';

import {
  ReadAllByMember,
  ReadAllByMemberSuccess,
  ReadAllByMemberFailure,
  ActionType,
} from './notification.action';

@Injectable()
export class Effects {

  constructor(
    private actions$: Actions,
    private notificationService: NotificationService,
    private router: Router
  ) { }

  @Effect()
  readAllByMember$: Observable<Action> = this.actions$
    // .ofType(ActionType.ReadAllByMember)
    // .map((action: ReadAllByMember) => action.payload)
    // .exhaustMap(member =>
    //   this.notificationService
    //   .readAllByMember(member)
    //     .map(notificationList => new ReadAllByMemberSuccess(notificationList))
    //     .catch(error => of(new ReadAllByMemberFailure(error)))
    // );
    .ofType(ActionType.ReadAllByMember)
    .map((action: ReadAllByMember) => action.payload)
    .switchMap(payload => this.notificationService.readAllByMember(payload.member, payload.pageParams))
    .map(notifications => {
      return new ReadAllByMemberSuccess(notifications);
    })
    .catch((error: RPCError) => {
      return of(new ReadAllByMemberFailure(error));
    });

}
